@@ -20,6 +20,9 @@ class Disk : public ALabel {
  private:
   util::SleeperThread thread_;
   std::string path_;
+  std::string unit_;
+
+  float calc_specific_divisor(const std::string divisor);
 };
 
 }  // namespace waybar::modules
