@@ -7,6 +7,7 @@
 #include <string>
 
 #include "ALabel.hpp"
+#include "gtkmm/box.h"
 #include "util/command.hpp"
 #include "util/json.hpp"
 #include "util/sleeper_thread.hpp"
@@ -15,18 +16,23 @@ namespace waybar::modules {
 
 class Image : public AModule {
  public:
-  Image(const std::string&, const std::string&, const Json::Value&);
-  auto update() -> void;
-  void refresh(int /*signal*/);
+  Image(const std::string&, const Json::Value&);
+  virtual ~Image() = default;
+  auto update() -> void override;
+  void refresh(int /*signal*/) override;
 
  private:
   void delayWorker();
   void handleEvent();
+  void parseOutputRaw();
 
+  Gtk::Box box_;
   Gtk::Image image_;
   std::string path_;
+  std::string tooltip_;
   int size_;
   int interval_;
+  util::command::res output_;
 
   util::SleeperThread thread_;
 };
