@@ -14,8 +14,8 @@ namespace waybar::modules {
 class JACK : public ALabel {
  public:
   JACK(const std::string &, const Json::Value &);
-  ~JACK() = default;
-  auto update() -> void;
+  virtual ~JACK() = default;
+  auto update() -> void override;
 
   int bufSize(jack_nframes_t size);
   int sampleRate(jack_nframes_t rate);
