@@ -42,7 +42,7 @@ auto Mode::update() -> void {
   if (mode_.empty()) {
     event_box_.hide();
   } else {
-    label_.set_markup(fmt::format(format_, mode_));
+    label_.set_markup(fmt::format(fmt::runtime(format_), mode_));
     if (tooltipEnabled()) {
       label_.set_tooltip_text(mode_);
     }
