@@ -2,8 +2,8 @@
 
 int ustring_clen(const Glib::ustring &str) {
   int total = 0;
-  for (auto i = str.begin(); i != str.end(); ++i) {
-    total += g_unichar_iswide(*i) + 1;
+  for (unsigned int i : str) {
+    total += g_unichar_iswide(i) + 1;
   }
   return total;
-}
\ No newline at end of file
+}
